@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2008-2013 CoNWeT Lab., Universidad Politécnica de Madrid
+# Copyright (c) 2008-2014 CoNWeT Lab., Universidad Politécnica de Madrid
 
 # This file is part of Wirecloud.
 
@@ -21,6 +21,8 @@ import os
 import sys
 import time
 
+from django.core.urlresolvers import reverse
+from django.utils.http import urlencode
 from django.utils.importlib import import_module
 from selenium.common.exceptions import StaleElementReferenceException
 from selenium.webdriver import ActionChains
@@ -37,6 +39,44 @@ def marketplace_loaded(driver):
     return False
 
 
+class PopupMenuTester(object):
+
+    def __init__(self, testcase, element):
+
+        self.testcase = testcase
+        self.element = element
+
+    def get_entry(self, name):
+
+        items = self.element.find_elements_by_css_selector('.menu_item')
+
+        for item in items:
+            span = item.find_element_by_css_selector('span')
+            if span and span.text == name:
+                return item
+
+        return None
+
+    def click_entry(self, name):
+        self.get_entry(name).click()
+
+    def check(self, must_be=(), must_be_absent=(), must_be_disabled=()):
+
+        for item in must_be:
+            menu_item = self.get_entry(item)
+            self.testcase.assertIsNotNone(menu_item, '"%(item)s" item should be present' % { 'item': item })
+            self.testcase.assertFalse('disabled' in menu_item.get_attribute('class'), '"%(item)s" item should be enabled' % { 'item': item })
+
+        for item in must_be_absent:
+            menu_item = self.get_entry(item)
+            self.testcase.assertIsNone(menu_item, '"%(item)s" item shouldn\'t be present' % { 'item': item })
+
+        for item in must_be_disabled:
+            menu_item = self.get_entry(item)
+            self.testcase.assertIsNotNone(menu_item, '"%(item)s" item should be present' % { 'item': item })
+            self.testcase.assertTrue('disabled' in menu_item.get_attribute('class'), '"%(item)s" item shouldn\'t be enabled' % { 'item': item })
+
+
 class WiringEndpointTester(object):
 
     def __init__(self, testcase, endpoint_name, element):
@@ -78,10 +118,7 @@ class IWidgetTester(object):
             return self.element
 
     def __enter__(self):
-        self.content_element = self.testcase.driver.execute_script('return opManager.activeWorkspace.getIWidget(%d).content;' % self.id)
-
-        # TODO work around webdriver bugs
-        self.testcase.driver.switch_to_default_content()
+        self.content_element = self.testcase.driver.execute_script('return Wirecloud.activeWorkspace.getIWidget(%d).content;' % self.id)
 
         self.testcase.driver.switch_to_frame(self.content_element)
         return None
@@ -102,7 +139,7 @@ class IWidgetTester(object):
 
         old_frame = driver.execute_script("return window.frameElement")
         driver.switch_to_default_content()
-        error_count = driver.execute_script('return opManager.activeWorkspace.getIWidget(%s).internal_iwidget.logManager.errorCount' % self.id)
+        error_count = driver.execute_script('return Wirecloud.activeWorkspace.getIWidget(%s).internal_iwidget.logManager.errorCount' % self.id)
         driver.switch_to_frame(old_frame)
 
         return error_count
@@ -114,7 +151,7 @@ class IWidgetTester(object):
         old_frame = driver.execute_script("return window.frameElement")
         driver.switch_to_default_content()
         log_entries = driver.execute_script('''
-            var iwidget = opManager.activeWorkspace.getIWidget(%s).internal_iwidget;
+            var iwidget = Wirecloud.activeWorkspace.getIWidget(%s).internal_iwidget;
             return iwidget.logManager.entries.map(function (entry) { return {date: entry.date.getTime(), level: entry.level, msg: entry.msg}; });
         ''' % self.id)
         driver.switch_to_frame(old_frame)
@@ -135,7 +172,7 @@ class IWidgetTester(object):
         self.element.find_element_by_css_selector('.statusBar').click()
 
         def name_changed(driver):
-            return driver.execute_script('return opManager.activeWorkspace.getIWidget(%s).name === "%s"' % (self.id, new_name))
+            return driver.execute_script('return Wirecloud.activeWorkspace.getIWidget(%s).name === "%s"' % (self.id, new_name))
 
         WebDriverWait(self.testcase.driver, timeout).until(name_changed)
 
@@ -143,7 +180,7 @@ class IWidgetTester(object):
 
         def iwidget_loaded(driver):
             iwidget_element = driver.execute_script('''
-                var iwidget = opManager.activeWorkspace.getIWidget(%s);
+                var iwidget = Wirecloud.activeWorkspace.getIWidget(%s);
                 return iwidget.internal_iwidget.loaded ? iwidget.element : null;
             ''' % self.id)
 
@@ -157,7 +194,7 @@ class IWidgetTester(object):
 
     def remove(self, timeout=30):
 
-        old_iwidget_ids = self.testcase.driver.execute_script('return opManager.activeWorkspace.getIWidgets().map(function(iwidget) {return iwidget.id;});')
+        old_iwidget_ids = self.testcase.driver.execute_script('return Wirecloud.activeWorkspace.getIWidgets().map(function(iwidget) {return iwidget.id;});')
         old_iwidget_count = len(old_iwidget_ids)
 
         self.element.find_element_by_css_selector('.icon-remove').click()
@@ -210,44 +247,24 @@ class IOperatorTester(object):
         ''' % {"ioperator": self.id, "endpoint": endpoint_name}
         ))
 
-class WirecloudRemoteTestCase(object):
+    @property
+    def error_count(self):
+        return self.testcase.driver.execute_script('return Wirecloud.activeWorkspace.wiring.ioperators[%s].logManager.errorCount' % self.id)
 
-    @classmethod
-    def setUpClass(cls):
+    @property
+    def log_entries(self):
+        return self.testcase.driver.execute_script('''
+            var ioperator = Wirecloud.activeWorkspace.wiring.ioperators[%s];
+            return ioperator.logManager.entries.map(function (entry) { return {date: entry.date.getTime(), level: entry.level, msg: entry.msg}; });
+        ''' % self.id)
 
-        cls.shared_test_data_dir = os.path.join(os.path.dirname(__file__), '../test-data')
-        cls.test_data_dir = os.path.join(os.path.dirname(sys.modules[cls.__module__].__file__), 'test-data')
 
-        # Load webdriver
-        module_name, klass_name = getattr(cls, '_webdriver_class', 'selenium.webdriver.Firefox').rsplit('.', 1)
-        module = import_module(module_name)
-        webdriver_args = getattr(cls, '_webdriver_args', None)
-        if webdriver_args is None:
-            webdriver_args = {}
-        cls.driver = getattr(module, klass_name)(**webdriver_args)
-
-    @classmethod
-    def tearDownClass(cls):
-
-        cls.driver.quit()
-
-    def tearDown(self):
-
-        self.driver.delete_all_cookies()
+class RemoteTestCase(object):
 
     def fill_form_input(self, form_input, value):
         # We cannot use send_keys due to http://code.google.com/p/chromedriver/issues/detail?id=35
         self.driver.execute_script('arguments[0].value = arguments[1]', form_input, value)
 
-    def scroll_and_click(self, element):
-
-        # Work around chrome and firefox driver bugs
-        try:
-            self.driver.execute_script("arguments[0].scrollIntoView(false);", element);
-        except:
-            pass
-        ActionChains(self.driver).click(element).perform()
-
     def wait_element_visible_by_css_selector(self, selector, timeout=30, element=None):
         if element is None:
             element = self.driver
@@ -272,6 +289,57 @@ class WirecloudRemoteTestCase(object):
         time.sleep(0.1)
         return element.find_element_by_xpath(selector)
 
+    def get_current_iwidgets(self, tab=None):
+
+        if tab is None:
+            iwidget_ids = self.driver.execute_script('return Wirecloud.activeWorkspace.getIWidgets().map(function(iwidget) {return iwidget.id;});')
+            iwidget_elements = self.driver.execute_script('return Wirecloud.activeWorkspace.getIWidgets().map(function(iwidget) {return iwidget.internal_iwidget.loaded ? iwidget.element : null;});')
+        else:
+            iwidget_ids = self.driver.execute_script('return Wirecloud.activeWorkspace.getTab(arguments[0]).getIWidgets().map(function(iwidget) {return iwidget.id;});', tab)
+            iwidget_elements = self.driver.execute_script('return Wirecloud.activeWorkspace.getTab(arguments[0]).getIWidgets().map(function(iwidget) {return iwidget.internal_iwidget.loaded ? iwidget.element : null;});', tab)
+
+        # Work around race condition reading iwidget ids and elements
+        if len(iwidget_ids) != len(iwidget_elements):
+            time.sleep(0.1)
+            return self.get_current_iwidgets(tab)
+
+        return [IWidgetTester(self, iwidget_ids[i], iwidget_elements[i]) for i in range(len(iwidget_ids))]
+
+
+class WirecloudRemoteTestCase(RemoteTestCase):
+
+    @classmethod
+    def setUpClass(cls):
+
+        cls.shared_test_data_dir = os.path.join(os.path.dirname(__file__), '../test-data')
+        cls.test_data_dir = os.path.join(os.path.dirname(sys.modules[cls.__module__].__file__), 'test-data')
+
+        # Load webdriver
+        module_name, klass_name = getattr(cls, '_webdriver_class', 'selenium.webdriver.Firefox').rsplit('.', 1)
+        module = import_module(module_name)
+        webdriver_args = getattr(cls, '_webdriver_args', None)
+        if webdriver_args is None:
+            webdriver_args = {}
+        cls.driver = getattr(module, klass_name)(**webdriver_args)
+
+    @classmethod
+    def tearDownClass(cls):
+
+        cls.driver.quit()
+
+    def tearDown(self):
+
+        self.driver.delete_all_cookies()
+
+    def scroll_and_click(self, element):
+
+        # Work around chrome and firefox driver bugs
+        try:
+            self.driver.execute_script("arguments[0].scrollIntoView(false);", element);
+        except:
+            pass
+        ActionChains(self.driver).click(element).perform()
+
     def wait_wirecloud_ready(self, start_timeout=30, timeout=30):
 
         loading_window = self.driver.find_element_by_css_selector('#loading-window')
@@ -298,12 +366,15 @@ class WirecloudRemoteTestCase(object):
         search_view = catalogue_element.find_element_by_class_name('search_interface')
         WebDriverWait(self.driver, timeout).until(lambda driver: 'disabled' not in search_view.get_attribute('class'))
 
-    def login(self, username='admin', password='admin'):
+    def login(self, username='admin', password='admin', next=None):
 
-        self.driver.get(self.live_server_url + '/login')
-        self.wait_element_visible_by_css_selector('#id_username')
+        url = self.live_server_url + '/login'
+        if next is not None:
+            url += '?next=' + next
 
-        username_input = self.driver.find_element_by_id('id_username')
+        self.driver.get(url)
+
+        username_input = self.wait_element_visible_by_css_selector('#id_username')
         self.fill_form_input(username_input, username)
         password_input = self.driver.find_element_by_id('id_password')
         self.fill_form_input(password_input, password)
@@ -330,20 +401,9 @@ class WirecloudRemoteTestCase(object):
 
     def check_popup_menu(self, must_be=(), must_be_absent=(), must_be_disabled=()):
 
-        time.sleep(0.1)
-
-        for item in must_be:
-            menu_item = self.get_popup_menu_item(item)
-            self.assertIsNotNone(menu_item)
-
-        for item in must_be_absent:
-            menu_item = self.get_popup_menu_item(item)
-            self.assertIsNone(menu_item)
-
-        for item in must_be_disabled:
-            menu_item = self.get_popup_menu_item(item)
-            self.assertIsNotNone(menu_item)
-            self.assertTrue('disabled' in menu_item.get_attribute('class'))
+        popup_menu_element = self.wait_element_visible_by_css_selector('.popup_menu')
+        tester = PopupMenuTester(self, popup_menu_element)
+        tester.check(must_be, must_be_absent, must_be_disabled)
 
     def add_packaged_resource_to_catalogue(self, wgt_file, widget_name, shared=False, expect_error=False):
 
@@ -447,12 +507,6 @@ class WirecloudRemoteTestCase(object):
 
         return None
 
-    def get_current_iwidgets(self):
-        iwidget_ids = self.driver.execute_script('return opManager.activeWorkspace.getIWidgets().map(function(iwidget) {return iwidget.id;});')
-        iwidget_elements = self.driver.execute_script('return opManager.activeWorkspace.getIWidgets().map(function(iwidget) {return iwidget.internal_iwidget.loaded ? iwidget.element : null;});')
-
-        return [IWidgetTester(self, iwidget_ids[i], iwidget_elements[i]) for i in range(len(iwidget_ids))]
-
     def get_current_wiring_editor_ioperators(self):
 
         ioperators = self.driver.execute_script('''
@@ -474,7 +528,7 @@ class WirecloudRemoteTestCase(object):
 
     def instantiate(self, resource, timeout=30):
 
-        old_iwidget_ids = self.driver.execute_script('return opManager.activeWorkspace.getIWidgets().map(function(iwidget) {return iwidget.id;});')
+        old_iwidget_ids = self.driver.execute_script('return Wirecloud.activeWorkspace.getIWidgets().map(function(iwidget) {return iwidget.id;});')
         old_iwidget_count = len(old_iwidget_ids)
 
         self.scroll_and_click(resource.find_element_by_css_selector('.instantiate_button div'))
@@ -571,21 +625,26 @@ class WirecloudRemoteTestCase(object):
     def count_iwidgets(self):
         return len(self.driver.find_elements_by_css_selector('div.iwidget'))
 
-    def get_popup_menu_item(self, item_name):
+    def popup_menu_click(self, item_name, element=None):
 
-        items = self.driver.find_elements_by_css_selector('.popup_menu > .menu_item')
-        for item in items:
-            span = item.find_element_by_css_selector('span')
-            if span and span.text == item_name:
-                return item
+        if element is None:
+            element = self.wait_element_visible_by_css_selector('.popup_menu')
 
-        return None
+        if not isinstance(item_name, tuple):
+            item_name = (item_name,)
 
-    def popup_menu_click(self, item_name):
+        tester = PopupMenuTester(self, element)
+        prev_popups = self.driver.find_elements_by_css_selector('.popup_menu')
+        for i, entry in enumerate(item_name[:-1]):
+            current_element = tester.get_entry(entry)
+            ActionChains(self.driver).move_to_element(current_element).perform()
+            WebDriverWait(self.driver, 5).until(lambda driver: len(self.driver.find_elements_by_css_selector('.popup_menu')) > len(prev_popups))
+            next_popups = self.driver.find_elements_by_css_selector('.popup_menu')
+            next_popup = next_popups[-1]
+            tester = PopupMenuTester(self, next_popup)
+            prev_popups = next_popups
 
-        self.wait_element_visible_by_css_selector('.popup_menu')
-        item = self.get_popup_menu_item(item_name)
-        item.click()
+        tester.click_entry(item_name[-1])
 
     def get_current_workspace_name(self):
 
@@ -708,13 +767,19 @@ class WirecloudRemoteTestCase(object):
         except:
             return self.driver.find_element_by_css_selector('#wirecloud_breadcrum .second_level').text
 
-    def perform_market_action(self, action):
+    def open_marketplace_menu(self):
+        self.change_main_view('marketplace')
         try:
             self.driver.find_element_by_css_selector('#wirecloud_breadcrum .second_level > .icon-menu').click()
         except:
             self.driver.find_element_by_css_selector('#wirecloud_breadcrum .third_level > .icon-menu').click()
 
-        self.popup_menu_click(action)
+        popup_menu_element = self.wait_element_visible_by_css_selector('.popup_menu')
+
+        return PopupMenuTester(self, popup_menu_element)
+
+    def perform_market_action(self, action):
+        self.open_marketplace_menu().click_entry(action)
 
     def perform_workspace_action(self, action):
         self.change_main_view('workspace')
@@ -734,7 +799,7 @@ class WirecloudRemoteTestCase(object):
 
         return None
 
-    def add_marketplace(self, name, url, type_, expect_error=False):
+    def add_marketplace(self, name, url, type_, expect_error=False, public=False):
 
         self.change_main_view('marketplace')
         self.perform_market_action("Add new marketplace")
@@ -746,6 +811,9 @@ class WirecloudRemoteTestCase(object):
         market_type_input = self.driver.find_element_by_css_selector('.window_menu .styled_form select')
         self.fill_form_input(market_type_input, type_)
 
+        if public:
+            self.driver.find_element_by_css_selector('.window_menu .styled_form input[name="public"]').click()
+
         self.driver.find_element_by_xpath("//*[contains(@class, 'window_menu')]//*[text()='Accept']").click()
         self.wait_wirecloud_ready()
         time.sleep(0.1)  # work around some problems
@@ -819,7 +887,7 @@ class WirecloudRemoteTestCase(object):
         resource = self.search_in_catalogue_results(resource_name)
         self.assertIsNone(resource)
 
-    def uninstall_resource(self, resource_name, timeout=30):
+    def uninstall_resource(self, resource_name, timeout=30, expect_error=False):
         self.change_main_view('marketplace')
         catalogue_base_element = self.get_current_catalogue_base_element()
 
@@ -830,16 +898,68 @@ class WirecloudRemoteTestCase(object):
         WebDriverWait(self.driver, timeout).until(lambda driver: catalogue_base_element.find_element_by_css_selector('.advanced_operations').is_displayed())
         time.sleep(0.1)
 
-        found = False
+        uninstall_button = None
         for operation in self.driver.find_elements_by_css_selector('.advanced_operations .styled_button'):
             if operation.text == 'Uninstall':
-                found = True
-                operation.find_element_by_css_selector('div').click()
+                uninstall_button = operation.find_element_by_css_selector('div')
                 break
-        self.assertTrue(found)
+
+        if expect_error:
+            self.assertIsNone(uninstall_button)
+        else:
+            self.assertIsNotNone(uninstall_button)
+            uninstall_button.click()
+
+            self.wait_wirecloud_ready()
+
+            self.search_resource(resource_name)
+            resource = self.search_in_catalogue_results(resource_name)
+            self.assertIsNone(resource)
+
+
+class MobileWirecloudRemoteTestCase(RemoteTestCase):
+
+    @classmethod
+    def setUpClass(cls):
+
+        cls.shared_test_data_dir = os.path.join(os.path.dirname(__file__), '../test-data')
+        cls.test_data_dir = os.path.join(os.path.dirname(sys.modules[cls.__module__].__file__), 'test-data')
+
+        # Load webdriver
+        module_name, klass_name = getattr(cls, '_webdriver_class', 'selenium.webdriver.Firefox').rsplit('.', 1)
+        module = import_module(module_name)
+        webdriver_args = getattr(cls, '_webdriver_args', None)
+        if webdriver_args is None:
+            webdriver_args = {}
+        cls.driver = getattr(module, klass_name)(**webdriver_args)
+
+    @classmethod
+    def tearDownClass(cls):
+
+        cls.driver.quit()
+
+    def login(self, username='admin', password='admin', next=None):
+
+        url = self.live_server_url
+        url += reverse('login')
+        if next is not None:
+            next_url = next
+        else:
+            next_url = self.live_server_url
+        next_url += "?mode=smartphone"
+        url += "?" + urlencode({'next': next_url})
+
+        self.driver.get(url)
+        self.wait_element_visible_by_css_selector('#id_username')
+
+        username_input = self.driver.find_element_by_id('id_username')
+        self.fill_form_input(username_input, username)
+        password_input = self.driver.find_element_by_id('id_password')
+        self.fill_form_input(password_input, password)
+        password_input.submit()
 
         self.wait_wirecloud_ready()
 
-        self.search_resource(resource_name)
-        resource = self.search_in_catalogue_results(resource_name)
-        self.assertIsNone(resource)
+    def wait_wirecloud_ready(self):
+
+        self.wait_element_visible_by_css_selector('.wirecloud_tab')
