@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2013 CoNWeT Lab., Universidad Politécnica de Madrid
+# Copyright (c) 2013-2014 CoNWeT Lab., Universidad Politécnica de Madrid
 
 # This file is part of Wirecloud.
 
@@ -17,11 +17,11 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with Wirecloud.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals
 
 import json
-import urlparse
 
+from django.utils import six
 from django.utils.translation import ugettext as _
 
 from wirecloud.commons.utils.template.base import is_valid_name, is_valid_vendor, is_valid_version, TemplateParseException
@@ -33,7 +33,45 @@ class JSONTemplateParser(object):
     def __init__(self, template, base=None):
 
         self.base = base
-        self._info = json.loads(template)
+        if isinstance(template, basestring):
+            self._info = json.loads(template)
+        elif isinstance(template, dict):
+            self._info = template
+        else:
+            raise TemplateParseException('Invalid input data')
+
+        if 'type' not in self._info:
+            raise TemplateParseException(_('Missing resource type.'))
+
+        if self._info['type'] not in ('widget', 'operator', 'mashup'):
+            raise TemplateParseException(_('Invalid resource type: %s') % self._info['type'])
+
+    def _check_array_fields(self, fields, place=None, required=False):
+
+        if place is None:
+            place = self._info
+
+        for field in fields:
+            if field not in place:
+                if required:
+                    raise TemplateParseException('Missing required field: %s' % field)
+
+                place[field] = []
+            elif not isinstance(place[field], (list, tuple)):
+                raise TemplateParseException('An array value was expected for the %s field' % field)
+
+    def _check_string_fields(self, fields, place=None, required=False):
+        if place is None:
+            place = self._info
+
+        for field in fields:
+            if field not in place:
+                if required:
+                    raise TemplateParseException('Missing required field: %s' % field)
+
+                place[field] = ''
+            elif not isinstance(place[field], six.string_types):
+                raise TemplateParseException('A string value was expected for the %s field' % field)
 
     def _add_translation_index(self, value, **kwargs):
         index = get_trans_index(value)
@@ -47,24 +85,29 @@ class JSONTemplateParser(object):
 
     def _init(self):
 
-        for field in ['doc_uri', 'image_uri', 'iphone_image_uri']:
-            if field not in self._info:
-                self._info[field] = ''
-
+        self._check_string_fields(('title', 'description', 'authors', 'doc', 'image', 'smartphoneimage', 'license', 'licenseurl'))
         if self._info['type'] == 'widget':
 
-            if self._info.get('code_content_type') is None:
+            self._check_string_fields(('code_url',), required=True)
+            self._check_array_fields(('preferences', 'properties'))
+
+            if self._info.get('code_content_type', None) is None:
                 self._info['code_content_type'] = 'text/html'
 
-            if self._info.get('code_charset') is None:
+            if self._info.get('code_charset', None) is None:
                 self._info['code_charset'] = 'utf-8'
 
+        if not 'wiring' in self._info:
+            self._info['wiring'] = {}
+
+        self._check_array_fields(('inputs', 'outputs'), place=self._info['wiring'])
+
         # Translations
         self._info['translation_index_usage'] = {}
         if 'translations' not in self._info:
             self._info['translations'] = {}
 
-        self._add_translation_index(self._info['display_name'], type='resource', field='display_name')
+        self._add_translation_index(self._info['title'], type='resource', field='title')
         self._add_translation_index(self._info['description'], type='resource', field='description')
 
         if self._info['type'] != 'mashup':
@@ -89,6 +132,18 @@ class JSONTemplateParser(object):
                 self._add_translation_index(output_endpoint['label'], type='vdef', variable=output_endpoint['name'], field='label')
                 self._add_translation_index(output_endpoint['description'], type='vdef', variable=output_endpoint['name'], field='description')
 
+    def get_resource_type(self):
+        return self._info['type']
+
+    def get_resource_name(self):
+        return self._info['name']
+
+    def get_resource_vendor(self):
+        return self._info['vendor']
+
+    def get_resource_version(self):
+        return self._info['version']
+
     def get_resource_info(self):
 
         if not is_valid_vendor(self._info['vendor']):
