@@ -19,7 +19,7 @@
  *
  */
 
-/*global CatalogueResource, CatalogueSearchView, Constants, Event, gettext, interpolate, LayoutManagerFactory, OpManagerFactory, Wirecloud, StyledElements*/
+/*global CatalogueResource, CatalogueSearchView, gettext, interpolate, LayoutManagerFactory, OpManagerFactory, Wirecloud, StyledElements*/
 
 (function () {
 
@@ -55,9 +55,9 @@
             var new_status = options.catalogue.buildStateData();
 
             if (out_alternative === this.viewsByName.initial) {
-                HistoryManager.replaceState(new_status);
+                Wirecloud.HistoryManager.replaceState(new_status);
             } else {
-                HistoryManager.pushState(new_status);
+                Wirecloud.HistoryManager.pushState(new_status);
             }
         }.bind(this));
 
@@ -93,18 +93,19 @@
                 dry_run: true,
                 onSuccess: next,
                 onFailure: function (msg, details) {
+                    var dialog;
                     if (details != null && 'missingDependencies' in details) {
                         // Show missing dependencies
-                        var dialog = new Wirecloud.ui.MissingDependenciesWindowMenu(next, details);
-                        dialog.show();
+                        dialog = new Wirecloud.ui.MissingDependenciesWindowMenu(next, details);
                     } else {
-                        LayoutManagerFactory.getInstance().showMessageMenu(msg, Constants.Logging.ERROR_MSG);
+                        dialog = new Wirecloud.ui.MessageWindowMenu(msg, Wirecloud.constants.LOGGING.ERROR_MSG);
                     }
+                    dialog.show();
                 }
             })
         } else {
-            var local_widget = Wirecloud.LocalCatalogue.getResourceId(resource.getURI());
-            OpManagerFactory.getInstance().addInstance(local_widget);
+            var local_widget = Wirecloud.LocalCatalogue.getResourceId(resource.uri);
+            Wirecloud.activeWorkspace.addInstance(local_widget);
         }
     };
 
@@ -142,8 +143,8 @@
             layoutManager._startComplexTask(gettext("Importing resource into local repository"), 3);
             layoutManager.logSubTask(gettext('Uploading resource'));
 
-            this.catalogue.addResourceFromURL(resource.getUriTemplate(), {
-                packaged: resource.isPackaged(),
+            this.catalogue.addResourceFromURL(resource.description_url, {
+                packaged: resource.packaged,
                 onSuccess: function () {
                     LayoutManagerFactory.getInstance().logSubTask(gettext('Resource installed successfully'));
                     LayoutManagerFactory.getInstance().logStep('');
@@ -154,7 +155,7 @@
                     catalogue_source.refresh_search_results();
                 }.bind(this),
                 onFailure: function (msg) {
-                    LayoutManagerFactory.getInstance().showMessageMenu(msg, Constants.Logging.ERROR_MSG);
+                    (new Wirecloud.ui.MessageWindowMenu(msg, Wirecloud.constants.LOGGING.ERROR_MSG)).show();
                     Wirecloud.GlobalLogManager.log(msg);
                 },
                 onComplete: function () {
@@ -185,7 +186,7 @@
                     }
                 }.bind(this),
                 onFailure: function (msg) {
-                    LayoutManagerFactory.getInstance().showMessageMenu(msg, Constants.Logging.ERROR_MSG);
+                    (new Wirecloud.ui.MessageWindowMenu(msg, Wirecloud.constants.LOGGING.ERROR_MSG)).show();
                     Wirecloud.GlobalLogManager.log(msg);
                 },
                 onComplete: function () {
@@ -210,9 +211,9 @@
 
     CatalogueView.prototype.ui_commands.showDetails = function showDetails(resource) {
         return function (e) {
-            Event.stop(e);
             this.viewsByName.details.paint(resource);
             this.alternatives.showAlternative(this.viewsByName.details);
+            this.viewsByName.details.repaint();
         }.bind(this);
     };
 
@@ -226,12 +227,8 @@
         }.bind(this);
 
         error_callback = function (msg) {
-            var logManager, layoutManager;
-
-            layoutManager = LayoutManagerFactory.getInstance();
-
-            layoutManager._notifyPlatformReady();
-            layoutManager.showMessageMenu(msg, Constants.Logging.ERROR_MSG);
+            LayoutManagerFactory.getInstance()._notifyPlatformReady();
+            (new Wirecloud.ui.MessageWindowMenu(msg, Wirecloud.constants.LOGGING.ERROR_MSG)).show();
         };
 
         doRequest = function () {
