@@ -1,5 +1,5 @@
 /*
- *     (C) Copyright 2012 Universidad Politécnica de Madrid
+ *     (C) Copyright 2012-2014 Universidad Politécnica de Madrid
  *
  *     This file is part of Wirecloud Platform.
  *
@@ -19,7 +19,7 @@
  *
  */
 
-/*global Constants, gettext, StyledElements, Wirecloud, LayoutManagerFactory*/
+/*global gettext, StyledElements, Wirecloud, LayoutManagerFactory*/
 
 (function () {
 
@@ -69,7 +69,6 @@
         if (!this.loading && !this.error) {
             items.push(new StyledElements.MenuItem(gettext('Add new marketplace'), function () {
                 var menu, fields, type_entries;
-                //type_entries = this.market.market_types;
 
                 fields = {
                     'name': {
@@ -90,6 +89,12 @@
                         'required': true
                     }
                 };
+                if (Wirecloud.contextManager.get('issuperuser')) {
+                    fields['public'] = {
+                        'type': 'boolean',
+                        'label': gettext('Public')
+                    }
+                }
                 menu = new Wirecloud.ui.FormWindowMenu(fields, gettext('Add Marketplace'));
 
                 // Form data is sent to server
@@ -100,9 +105,13 @@
                             "name": data.name,
                             "url": data.url,
                             "type": data.type,
-                            "user": OpManagerFactory.getInstance().contextManager.get('username')
                         }
                     };
+                    if (data['public'] === true) {
+                        market_info.options['user'] = null;
+                    } else {
+                        market_info.options['user'] = Wirecloud.contextManager.get('username');
+                    }
                     Wirecloud.MarketManager.addMarket(market_info, this.market.addMarket.bind(this.market, market_info.options));
                 }.bind(this);
 
@@ -129,7 +138,7 @@
                                 });
                             }.bind(this),
                             onFailure: function (msg) {
-                                LayoutManagerFactory.getInstance().showMessageMenu(msg, Constants.Logging.ERROR_MSG);
+                                (new Wirecloud.ui.MessageWindowMenu(msg, Wirecloud.constants.LOGGING.ERROR_MSG)).show();
                                 Wirecloud.GlobalLogManager.log(msg);
                                 LayoutManagerFactory.getInstance()._notifyPlatformReady();
                             }
