@@ -1,26 +1,24 @@
 /*
- *     Copyright (c) 2012-2013 CoNWeT Lab., Universidad Politécnica de Madrid
- *     Copyright (c) 2012-2013 Center for Open Middleware
+ *     DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER
  *
- *     This file is part of Wirecloud Platform.
+ *     Copyright (c) 2012-2014 Universidad Politécnica de Madrid
+ *     Copyright (c) 2012-2014 the Center for Open Middleware
  *
- *     Wirecloud Platform is free software: you can redistribute it and/or
- *     modify it under the terms of the GNU Affero General Public License as
- *     published by the Free Software Foundation, either version 3 of the
- *     License, or (at your option) any later version.
+ *     Licensed under the Apache License, Version 2.0 (the
+ *     "License"); you may not use this file except in compliance
+ *     with the License.  You may obtain a copy of the License at
  *
- *     Wirecloud is distributed in the hope that it will be useful, but WITHOUT
- *     ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- *     FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
- *     License for more details.
- *
- *     You should have received a copy of the GNU Affero General Public License
- *     along with Wirecloud Platform.  If not, see
- *     <http://www.gnu.org/licenses/>.
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
+ *     Unless required by applicable law or agreed to in writing,
+ *     software distributed under the License is distributed on an
+ *     "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *     KIND, either express or implied.  See the License for the
+ *     specific language governing permissions and limitations
+ *     under the License.
  */
 
-/*global opManager, Variable, Wirecloud */
+/*global opManager, Wirecloud */
 
 (function () {
 
@@ -70,7 +68,7 @@
             isGhost = false;
         }
 
-        Wirecloud.ui.WiringEditor.GenericInterface.call(this, false, wiringEditor, this.iwidget.name, manager, 'iwidget', isGhost);
+        Wirecloud.ui.WiringEditor.GenericInterface.call(this, false, wiringEditor, iwidget, this.iwidget.name, manager, 'iwidget', isGhost);
         if (!isMenubarRef) {
 
             // Sort
@@ -118,6 +116,7 @@
 
         if (!this.wiringEditor.withinGrid(e)) {
             this.wiringEditor.layout.wrapperElement.removeChild(data.iObjectClon.wrapperElement);
+            this.wiringEditor.events.widgetaddfail.dispatch();
             return;
         }
 
