@@ -26,13 +26,23 @@
     "use strict";
 
     var Operator = function Operator(operator_meta, id, operator_status, /* TODO */ wiringEditor) {
-        var i, inputs, outputs, data_uri, preferences, key;
+        var i, inputs, outputs, data_uri, preferenceList, preferences, readonly, hidden, key, value;
+
+        if (["object", "undefined"].indexOf(typeof operator_status) === -1) {
+            throw new TypeError('invalid operator status');
+        }
+        if (operator_status == null) {
+            operator_status = {};
+        }
+        if (operator_status.preferences == null) {
+            operator_status.preferences = {};
+        }
 
         StyledElements.ObjectWithEvents.call(this, ['load', 'unload']);
 
         Object.defineProperty(this, 'meta', {value: operator_meta});
         Object.defineProperty(this, 'name', {value: operator_meta.name});
-        Object.defineProperty(this, 'display_name', {value: operator_meta.display_name});
+        Object.defineProperty(this, 'title', {value: operator_meta.title});
         Object.defineProperty(this, 'id', {value: "" + id});
         Object.defineProperty(this, 'logManager', {value: new Wirecloud.wiring.OperatorLogManager(this)});
 
@@ -52,29 +62,32 @@
 
         this.pending_events = [];
 
+        preferenceList = [];
         preferences = {};
-        if (operator_status) {
-            for (key in operator_status.preferences) {
-                preferences[key] = operator_status.preferences[key];
-            }
-        }
-
-        for (key in this.meta.preferences) {
-            if (!(key in preferences)) {
-                preferences[key] = {
-                    'readonly': false,
-                    'hidden': false,
-                    'value': this.meta.preferences[key].default_value
-                };
+        for (i = 0; i < this.meta.preferenceList.length; i++) {
+            key = this.meta.preferenceList[i].name;
+            if (key in operator_status.preferences) {
+                value = operator_status.preferences[key].value;
+                readonly = operator_status.preferences[key].readonly;
+                hidden = operator_status.preferences[key].hidden;
+            } else {
+                value = this.meta.preferenceList[i].default;
+                readonly = hidden = false;
             }
+            preferences[key] = new Wirecloud.UserPref(this.meta.preferenceList[i], readonly, hidden, value);
+            preferenceList.push(preferences[key]);
         }
+        Object.freeze(preferenceList);
+        Object.defineProperty(this, 'preferenceList', {value: preferenceList});
         Object.freeze(preferences);
         Object.defineProperty(this, 'preferences', {value: preferences});
 
         if (!wiringEditor) {
-            this.element = document.createElement('object');
-            data_uri = Wirecloud.URLs.OPERATOR_ENTRY.evaluate({vendor: operator_meta.vendor, name: operator_meta.name, version: operator_meta.version}) + '#id=' + id;
+            this.element = document.createElement('iframe');
+            this.element.className = 'ioperator';
+            data_uri = Wirecloud.URLs.OPERATOR_ENTRY.evaluate({vendor: operator_meta.vendor, name: operator_meta.name, version: operator_meta.version.text}) + '#id=' + id;
             this.element.addEventListener('load', function () {
+                this.logManager.log('Operator loaded', Wirecloud.constants.LOGGING.INFO_MSG);
                 this.loaded = true;
                 this.events.load.dispatch(this);
                 for (var i = 0; i < this.pending_events.length; i += 1) {
@@ -83,15 +96,21 @@
                 this.pending_events = [];
             }.bind(this), true);
             this.element.addEventListener('unload', function () {
+                this.logManager.log('Operator unloaded', Wirecloud.constants.LOGGING.INFO_MSG);
                 this.loaded = false;
                 this.events.unload.dispatch(this);
             }.bind(this), true);
-            this.element.setAttribute('data', data_uri);
-            document.body.appendChild(this.element);
+            this.element.setAttribute('type', 'text/html');
+            this.element.setAttribute('src', data_uri);
+            document.getElementById('workspace').appendChild(this.element);
         }
     };
     Operator.prototype = new StyledElements.ObjectWithEvents();
 
+    Operator.prototype.registerPrefCallback = function registerPrefCallback(prefCallback) {
+        this.prefCallback = prefCallback;
+    };
+
     Operator.prototype.fullDisconnect = function fullDisconnect() {
         var key, connectables;
 
