@@ -1,23 +1,22 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2013 Universidad Politécnica de Madrid
+# Copyright (c) 2013 CoNWeT Lab., Universidad Politécnica de Madrid
 
 # This file is part of Wirecloud.
 
 # Wirecloud is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
+# it under the terms of the GNU Affero General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 
 # Wirecloud is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Affero General Public License for more details.
 
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Affero General Public License
 # along with Wirecloud.  If not, see <http://www.gnu.org/licenses/>.
 
-
 import json
 
 from django.http import HttpResponse
@@ -54,7 +53,7 @@ class ThemeEntry(Resource):
 
         context = RequestContext(request)
         templates = {}
-        template_descriptions = get_templates('index')
+        template_descriptions = get_templates('classic')
         for template_id in template_descriptions:
             template = get_template(template_descriptions[template_id])
             templates[template_id] = template.render(context)
