@@ -1,25 +1,22 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2012-2013 Universidad Politécnica de Madrid
+# Copyright (c) 2012-2013 CoNWeT Lab., Universidad Politécnica de Madrid
 
 # This file is part of Wirecloud.
 
 # Wirecloud is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
+# it under the terms of the GNU Affero General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 
 # Wirecloud is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Affero General Public License for more details.
 
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Affero General Public License
 # along with Wirecloud.  If not, see <http://www.gnu.org/licenses/>.
 
-
-# -*- coding: utf-8 -*-
-
 import os.path
 import re
 
@@ -82,30 +79,42 @@ class CodeTransformationTestCase(TestCase):
 
     def test_basic_html(self):
         initial_code = self.read_file('test-data/xhtml1-initial.html')
-        final_code = self.XML_NORMALIZATION_RE.sub('><', fix_widget_code(initial_code, 'http://server.com/widget', 'text/html', None, 'utf-8', False)) + '\n'
+        final_code = self.XML_NORMALIZATION_RE.sub('><', fix_widget_code(initial_code, 'http://server.com/widget', 'text/html', None, 'utf-8', False, {}, False, 'classic')) + '\n'
         expected_code = self.read_file('test-data/xhtml1-expected.html')
         self.assertEqual(final_code, expected_code)
 
     def test_basic_html_iso8859_15(self):
         initial_code = self.read_file('test-data/xhtml1-iso8859-15-initial.html')
-        final_code = self.XML_NORMALIZATION_RE.sub('><', fix_widget_code(initial_code, 'http://server.com/widget', 'text/html', None, 'iso-8859-15', False)) + '\n'
+        final_code = self.XML_NORMALIZATION_RE.sub('><', fix_widget_code(initial_code, 'http://server.com/widget', 'text/html', None, 'iso-8859-15', False, {}, False, 'classic')) + '\n'
         expected_code = self.read_file('test-data/xhtml1-iso8859-15-expected.html')
         self.assertEqual(final_code, expected_code)
 
+    def test_html_with_one_base_element(self):
+        initial_code = self.read_file('test-data/xhtml4-initial.html')
+        final_code = self.XML_NORMALIZATION_RE.sub('><', fix_widget_code(initial_code, 'http://server.com/widget', 'text/html', None, 'utf-8', False, {}, False, 'classic')) + '\n'
+        expected_code = self.read_file('test-data/xhtml4-expected.html')
+        self.assertEqual(final_code, expected_code)
+
+    def test_html_with_more_than_one_base_element(self):
+        initial_code = self.read_file('test-data/xhtml4-extra-base-elements-initial.html')
+        final_code = self.XML_NORMALIZATION_RE.sub('><', fix_widget_code(initial_code, 'http://server.com/widget', 'text/html', None, 'utf-8', False, {}, False, 'classic')) + '\n'
+        expected_code = self.read_file('test-data/xhtml4-expected.html')
+        self.assertEqual(final_code, expected_code)
+
     def test_basic_xhtml(self):
         initial_code = self.read_file('test-data/xhtml2-initial.html')
-        final_code = self.XML_NORMALIZATION_RE.sub('><', fix_widget_code(initial_code, 'http://server.com/widget', 'application/xhtml+xml', None, 'utf-8', False)) + '\n'
+        final_code = self.XML_NORMALIZATION_RE.sub('><', fix_widget_code(initial_code, 'http://server.com/widget', 'application/xhtml+xml', None, 'utf-8', False, {}, False, 'classic')) + '\n'
         expected_code = self.read_file('test-data/xhtml2-expected.html')
         self.assertEqual(final_code, expected_code)
 
     def test_basic_xhtml_iso8859_15(self):
         initial_code = self.read_file('test-data/xhtml2-iso8859-15-initial.html')
-        final_code = self.XML_NORMALIZATION_RE.sub('><', fix_widget_code(initial_code, 'http://server.com/widget', 'application/xhtml+xml', None, 'iso-8859-15', False)) + '\n'
+        final_code = self.XML_NORMALIZATION_RE.sub('><', fix_widget_code(initial_code, 'http://server.com/widget', 'application/xhtml+xml', None, 'iso-8859-15', False, {}, False, 'classic')) + '\n'
         expected_code = self.read_file('test-data/xhtml2-iso8859-15-expected.html')
         self.assertEqual(final_code, expected_code)
 
     def test_xhtml_without_head_element(self):
         initial_code = self.read_file('test-data/xhtml3-initial.html')
-        final_code = self.XML_NORMALIZATION_RE.sub('><', fix_widget_code(initial_code, 'http://server.com/widget', 'application/xhtml+xml', None, 'utf-8', False)) + '\n'
+        final_code = self.XML_NORMALIZATION_RE.sub('><', fix_widget_code(initial_code, 'http://server.com/widget', 'application/xhtml+xml', None, 'utf-8', False, {}, False, 'classic')) + '\n'
         expected_code = self.read_file('test-data/xhtml3-expected.html')
         self.assertEqual(final_code, expected_code)
