@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2012-2013 Universidad Politécnica de Madrid
+# Copyright 2012-2014 (c) CoNWeT Lab., Universidad Politécnica de Madrid
 
 # This file is part of Wirecloud.
 
@@ -22,31 +22,26 @@ from cStringIO import StringIO
 import os
 import zipfile
 
-from django.contrib.auth.models import Group, User
-from django.core.urlresolvers import reverse
 from django.db import IntegrityError
-from django.http import HttpResponse, HttpResponseForbidden, Http404
+from django.http import HttpResponse
 from django.shortcuts import get_object_or_404
 from django.utils.translation import ugettext as _
-from django.utils.http import urlencode
 
 from wirecloud.catalogue import utils as catalogue
 from wirecloud.catalogue.models import CatalogueResource
 from wirecloud.commons.baseviews import Resource, Service
-from wirecloud.commons.utils import downloader
 from wirecloud.commons.utils.cache import no_cache
 from wirecloud.commons.utils.db import save_alternative
-from wirecloud.commons.utils.http import authentication_required, build_error_response, get_content_type, supported_request_mime_types, supported_response_mime_types
+from wirecloud.commons.utils.downloader import download_http_content
+from wirecloud.commons.utils.http import authentication_required, authentication_required_cond, build_error_response, get_content_type, supported_request_mime_types, supported_response_mime_types
 from wirecloud.commons.utils.template import is_valid_name, is_valid_vendor, is_valid_version, TemplateParser
 from wirecloud.commons.utils.transaction import commit_on_http_success
 from wirecloud.commons.utils.wgt import WgtFile
 from wirecloud.platform.get_data import get_workspace_data, get_global_workspace_data
-from wirecloud.platform.iwidget.utils import deleteIWidget
 from wirecloud.platform.models import IWidget, Tab, UserWorkspace, Workspace
-from wirecloud.platform.workspace.mashupTemplateGenerator import build_rdf_template_from_workspace
+from wirecloud.platform.settings import ALLOW_ANONYMOUS_ACCESS
+from wirecloud.platform.workspace.mashupTemplateGenerator import build_json_template_from_workspace, build_rdf_template_from_workspace
 from wirecloud.platform.workspace.mashupTemplateParser import check_mashup_dependencies, buildWorkspaceFromTemplate, fillWorkspaceUsingTemplate, MissingDependencies
-from wirecloud.platform.workspace.packageCloner import PackageCloner
-from wirecloud.platform.workspace.packageLinker import PackageLinker
 from wirecloud.platform.workspace.utils import deleteTab, createTab, get_workspace_list, setVisibleTab, set_variable_value
 from wirecloud.platform.markets.utils import get_market_managers
 
@@ -79,18 +74,6 @@ def setActiveWorkspace(user, workspace):
     UserWorkspace.objects.filter(user=user, workspace=workspace).update(active=True)
 
 
-def linkWorkspaceObject(user, workspace, creator, link_variable_values=True):
-    packageLinker = PackageLinker()
-
-    return packageLinker.link_workspace(workspace, user, creator, link_variable_values)
-
-
-def linkWorkspace(user, workspace_id, creator, link_variable_values=True):
-    workspace = get_object_or_404(Workspace, id=workspace_id)
-
-    return linkWorkspaceObject(user, workspace, creator, link_variable_values)
-
-
 def normalize_boolean_param(name, value):
 
     if isinstance(value, basestring):
@@ -106,16 +89,15 @@ def normalize_boolean_param(name, value):
 
 class WorkspaceCollection(Resource):
 
-    @authentication_required
     @supported_response_mime_types(('application/json',))
     @commit_on_http_success
     @no_cache
     def read(self, request):
 
-        workspaces, _junk, reload_showcase = get_workspace_list(request.user)
+        workspaces, _junk = get_workspace_list(request.user)
         data_list = [get_workspace_data(workspace, request.user) for workspace in workspaces]
 
-        return HttpResponse(json.dumps(data_list), mimetype='application/json; charset=UTF-8')
+        return HttpResponse(json.dumps(data_list), content_type='application/json; charset=UTF-8')
 
     @authentication_required
     @supported_request_mime_types(('application/json',))
@@ -124,23 +106,29 @@ class WorkspaceCollection(Resource):
     def create(self, request):
 
         try:
-            data = json.loads(request.raw_post_data)
-        except ValueError, e:
+            data = json.loads(request.body)
+        except ValueError as e:
             msg = _("malformed json data: %s") % unicode(e)
             return build_error_response(request, 400, msg)
 
         workspace_name = data.get('name', '').strip()
+        workspace_id = data.get('workspace', '')
         mashup_id = data.get('mashup', '')
         try:
             allow_renaming = normalize_boolean_param('allow_renaming', data.get('allow_renaming', False))
             dry_run = normalize_boolean_param('allow_renaming', data.get('dry_run', False))
-        except (TypeError, ValueError), e:
+        except (TypeError, ValueError) as e:
             return build_error_response(request, 422, unicode(e))
 
-        if mashup_id == '' and workspace_name == '':
-            return build_error_response(request, 422, _('missing workspace name'))
+        if mashup_id == '' and workspace_id == '' and workspace_name == '':
+            return build_error_response(request, 422, _('Missing name parameter'))
+        elif  mashup_id != '' and workspace_id != '':
+            return build_error_response(request, 422, _('Workspace and mashup parameters cannot be used at the same time'))
 
-        if mashup_id == '':
+        if mashup_id == '' and workspace_id == '':
+
+            if not is_valid_name(workspace_name):
+                return build_error_response(request, 422, _('invalid workspace name'))
 
             if dry_run:
                 return HttpResponse(status=204)
@@ -151,32 +139,51 @@ class WorkspaceCollection(Resource):
                 msg = _('A workspace with the given name already exists')
                 return build_error_response(request, 409, msg)
         else:
-            values = mashup_id.split('/', 3)
-            if len(values) != 3:
-                return build_error_response(request, 422, _('invalid mashup id'))
 
-            (mashup_vendor, mashup_name, mashup_version) = values
-            try:
-                resource = CatalogueResource.objects.get(vendor=mashup_vendor, short_name=mashup_name, version=mashup_version)
-                if not resource.is_available_for(request.user) or resource.resource_type() != 'mashup':
-                    raise CatalogueResource.DoesNotExist
-            except CatalogueResource.DoesNotExist:
-                return build_error_response(request, 422, _('Mashup not found: %(mashup_id)s') % {'mashup_id': mashup_id})
+            if mashup_id != '':
+                values = mashup_id.split('/', 3)
+                if len(values) != 3:
+                    return build_error_response(request, 422, _('invalid mashup id'))
 
-            if resource.fromWGT:
-                base_dir = catalogue.wgt_deployer.get_base_dir(mashup_vendor, mashup_name, mashup_version)
-                wgt_file = WgtFile(os.path.join(base_dir, resource.template_uri))
-                template = TemplateParser(wgt_file.get_template())
-            else:
-                template = downloader.download_http_content(resource.template_uri, user=request.user)
+                (mashup_vendor, mashup_name, mashup_version) = values
                 try:
-                    template = TemplateParser(template)
-                except:
-                    build_error_response(request, 424, _('Downloaded invalid resource description from: %(url)s') % {'url': resource.template_uri})
+                    resource = CatalogueResource.objects.get(vendor=mashup_vendor, short_name=mashup_name, version=mashup_version)
+                    if not resource.is_available_for(request.user) or resource.resource_type() != 'mashup':
+                        raise CatalogueResource.DoesNotExist
+                except CatalogueResource.DoesNotExist:
+                    return build_error_response(request, 422, _('Mashup not found: %(mashup_id)s') % {'mashup_id': mashup_id})
+
+                if resource.fromWGT:
+                    base_dir = catalogue.wgt_deployer.get_base_dir(mashup_vendor, mashup_name, mashup_version)
+                    wgt_file = WgtFile(os.path.join(base_dir, resource.template_uri))
+                    template = TemplateParser(wgt_file.get_template())
+                else:
+                    template = download_http_content(resource.template_uri, user=request.user)
+                    try:
+                        template = TemplateParser(template)
+                    except:
+                        build_error_response(request, 422, _('Downloaded invalid resource description from: %(url)s') % {'url': resource.template_uri})
+
+            else:
+
+                from_ws = get_object_or_404(Workspace, id=workspace_id)
+                if from_ws.public is False and not request.user.is_superuser and from_ws.creator != request.user:
+                    return build_error_response(request, 403, _('You are not allowed to read from workspace %s') % workspace_id)
+
+                options = {
+                    'vendor': 'api',
+                    'name': from_ws.name,
+                    'version': '1.0',
+                    'title': '',
+                    'description': 'Temporal mashup for the workspace copy operation',
+                    'email': 'a@example.com',
+                }
+
+                template = TemplateParser(build_json_template_from_workspace(options, from_ws, from_ws.creator))
 
             try:
                 check_mashup_dependencies(template, request.user)
-            except MissingDependencies, e:
+            except MissingDependencies as e:
                 details = {
                     'missingDependencies': e.missing_dependencies,
                 }
@@ -185,8 +192,11 @@ class WorkspaceCollection(Resource):
             if dry_run:
                 return HttpResponse(status=204)
 
+            if workspace_name == '':
+                workspace_name = None
+
             try:
-                workspace, _junk = buildWorkspaceFromTemplate(template, request.user, allow_renaming=allow_renaming)
+                workspace, _junk = buildWorkspaceFromTemplate(template, request.user, allow_renaming=allow_renaming, new_name=workspace_name)
             except IntegrityError:
                 msg = _('A workspace with the given name already exists')
                 return build_error_response(request, 409, msg)
@@ -198,14 +208,14 @@ class WorkspaceCollection(Resource):
 
 class WorkspaceEntry(Resource):
 
-    @authentication_required
+    @authentication_required_cond(ALLOW_ANONYMOUS_ACCESS)
     @supported_response_mime_types(('application/json',))
     def read(self, request, workspace_id):
 
         workspace = get_object_or_404(Workspace, pk=workspace_id)
 
-        if not workspace.users.filter(pk=request.user.pk).exists():
-            return HttpResponseForbidden()
+        if not workspace.public and not (request.user.is_authenticated() and workspace.users.filter(pk=request.user.pk).exists()):
+            return build_error_response(request, 403, _("You don't have permission to access this workspace"))
 
         workspace_data = get_global_workspace_data(workspace, request.user)
 
@@ -217,14 +227,14 @@ class WorkspaceEntry(Resource):
     def create(self, request, workspace_id):
 
         try:
-            ts = json.loads(request.raw_post_data)
-        except ValueError, e:
+            ts = json.loads(request.body)
+        except ValueError as e:
             msg = _("malformed json data: %s") % unicode(e)
             return build_error_response(request, 400, msg)
 
         workspace = Workspace.objects.get(pk=workspace_id)
         if not (request.user.is_superuser or workspace.users.filter(pk=request.user.pk).exists()):
-            return HttpResponseForbidden()
+            return build_error_response(request, 403, _('You are not allowed to update this workspace'))
 
         if 'active' in ts:
 
@@ -253,22 +263,15 @@ class WorkspaceEntry(Resource):
 
         workspace = get_object_or_404(Workspace, pk=workspace_id)
         if not workspace.users.filter(pk=request.user.pk).exists():
-            return HttpResponseForbidden()
+            return build_error_response(request, 403, _('You are not allowed to delete this workspace'))
 
-        user_workspace = UserWorkspace.objects.get(user=request.user, workspace=workspace)
-        # Check if the user does not have any other workspace
-        if user_workspace.manager != '' or UserWorkspace.objects.filter(user=request.user).count() == 0:
-            msg = _("workspace cannot be deleted")
-
-            return build_error_response(request, 403, msg)
-
-        user_workspace.delete()
+        UserWorkspace.objects.filter(user=request.user, workspace=workspace).delete()
         if workspace.users.count() == 0:
 
             # Remove the workspace
             iwidgets = IWidget.objects.filter(tab__workspace=workspace)
             for iwidget in iwidgets:
-                deleteIWidget(iwidget, request.user)
+                iwidget.delete()
             workspace.delete()
 
             from wirecloud.platform.get_data import _invalidate_cached_variable_values
@@ -286,8 +289,8 @@ class TabCollection(Resource):
     def create(self, request, workspace_id):
 
         try:
-            data = json.loads(request.raw_post_data)
-        except ValueError, e:
+            data = json.loads(request.body)
+        except ValueError as e:
             msg = _("malformed json data: %s") % unicode(e)
             return build_error_response(request, 400, msg)
 
@@ -297,7 +300,7 @@ class TabCollection(Resource):
         tab_name = data['name']
         workspace = Workspace.objects.get(pk=workspace_id)
         if not (request.user.is_superuser or workspace.creator == request.user):
-            return HttpResponseForbidden()
+            return build_error_response(request, 403, _('You are not allowed to create new tabs for this workspace'))
 
         try:
             tab = createTab(tab_name, workspace)
@@ -308,26 +311,23 @@ class TabCollection(Resource):
         # Returning created Ids
         ids = {'id': tab.id, 'name': tab.name}
 
-        return HttpResponse(json.dumps(ids), status=201, mimetype='application/json; charset=UTF-8')
+        return HttpResponse(json.dumps(ids), status=201, content_type='application/json; charset=UTF-8')
+
+
+class TabOrderService(Service):
 
     @authentication_required
     @supported_request_mime_types(('application/json',))
     @commit_on_http_success
-    def update(self, request, workspace_id):
+    def process(self, request, workspace_id):
 
-        user_workspaces = UserWorkspace.objects.select_related('workspace')
-        try:
-            user_workspace = user_workspaces.get(user__id=request.user.id, workspace__id=workspace_id)
-        except UserWorkspace.DoesNotExist:
-            raise Http404
-
-        workspace = user_workspace.workspace
-        if workspace.creator != request.user or user_workspace.manager != '':
-            return HttpResponseForbidden()
+        workspace = Workspace.objects.get(pk=workspace_id)
+        if not (request.user.is_superuser or workspace.creator == request.user):
+            return build_error_response(request, 403, _('You are not allowed to create new tabs for this workspace'))
 
         try:
-            order = json.loads(request.raw_post_data)
-        except ValueError, e:
+            order = json.loads(request.body)
+        except ValueError as e:
             msg = _("malformed json data: %s") % unicode(e)
             return build_error_response(request, 400, msg)
 
@@ -349,15 +349,15 @@ class TabEntry(Resource):
 
         tab = get_object_or_404(Tab.objects.select_related('workspace'), workspace__pk=workspace_id, pk=tab_id)
         if tab.workspace.creator != request.user:
-            return HttpResponseForbidden()
+            return build_error_response(request, 403, _('You are not allowed to update this workspace'))
 
         user_workspace = UserWorkspace.objects.get(user__id=request.user.id, workspace__id=workspace_id)
         if user_workspace.manager != '':
-            return HttpResponseForbidden()
+            return build_error_response(request, 403, _('You are not allowed to update this workspace'))
 
         try:
-            data = json.loads(request.raw_post_data)
-        except ValueError, e:
+            data = json.loads(request.body)
+        except ValueError as e:
             msg = _("malformed json data: %s") % unicode(e)
             return build_error_response(request, 400, msg)
 
@@ -391,12 +391,16 @@ class TabEntry(Resource):
         # Get tab, if it does not exist, an http 404 error is returned
         tab = get_object_or_404(Tab.objects.select_related('workspace'), workspace__pk=workspace_id, pk=tab_id)
         if not request.user.is_superuser and not tab.workspace.users.filter(id=request.user.id).exists():
-            return HttpResponseForbidden()
+            return build_error_response(request, 403, _('You are not allowed to remove this tab'))
 
         tabs = Tab.objects.filter(workspace__pk=workspace_id).order_by('position')[::1]
         if len(tabs) == 1:
-            msg = _("tab cannot be deleted")
-            return HttpResponseForbidden(msg)
+            msg = _("Tab cannot be deleted as workspaces need at least one tab")
+            return build_error_response(request, 403, msg)
+
+        if tab.iwidget_set.filter(readOnly=True).exists():
+            msg = _("Tab cannot be deleted as it contains widgets that cannot be deleted")
+            return build_error_response(request, 403, msg)
 
         # decrease the position of the following tabs
         for t in range(tab.position + 1, len(tabs)):
@@ -423,64 +427,17 @@ class WorkspaceVariableCollection(Resource):
     def create(self, request, workspace_id):
 
         try:
-            iwidgetVariables = json.loads(request.raw_post_data)
-        except ValueError, e:
+            iwidgetVariables = json.loads(request.body)
+        except ValueError as e:
             msg = _("malformed json data: %s") % unicode(e)
             return build_error_response(request, 400, msg)
 
         for igVar in iwidgetVariables:
-            set_variable_value(igVar['id'], request.user, igVar['value'])
+            set_variable_value(igVar['id'], igVar['value'])
 
         return HttpResponse(status=204)
 
 
-class WorkspaceSharerEntry(Resource):
-
-    @authentication_required
-    @commit_on_http_success
-    def update(self, request, workspace_id, share_boolean):
-
-        workspace = get_object_or_404(Workspace.objects.get(id=workspace_id))
-        owner = workspace.creator
-
-        if owner != request.user:
-            msg = 'You are not the owner of the workspace, so you can not share it!'
-            result = {'result': 'error', 'description': msg}
-            return HttpResponseForbidden(json.dumps(result), mimetype='application/json; charset=UTF-8')
-
-        #Everything right!
-        if request.raw_post_data == '':
-            #Share with everybody
-            #Linking with public user!
-            public_user = None  # TODO
-
-            linkWorkspaceObject(public_user, workspace, owner, link_variable_values=True)
-
-            workspace_path = reverse('wirecloud.workspace_view', args=(workspace_id,))
-            url = request.build_absolute_uri(workspace_path + '?' + urlencode({u'view': 'viewer'}))
-
-            result = {"result": "ok", "url": url}
-            return HttpResponse(json.dumps(result), mimetype='application/json; charset=UTF-8')
-        else:
-            #Share only with the scpecified groups
-            try:
-                groups = json.loads(request.raw_post_data)
-            except ValueError, e:
-                msg = _("malformed json data: %s") % unicode(e)
-                return build_error_response(request, 400, msg)
-
-            queryGroups = Group.objects.filter(id__in=groups)
-            for g in queryGroups:
-                workspace.targetOrganizations.add(g)
-
-            users = User.objects.filter(groups__in=groups).distinct()
-            for user in users:
-                #link the workspace with each user
-                linkWorkspaceObject(user, workspace, owner, link_variable_values=True)
-
-            return HttpResponse(status=204)
-
-
 class MashupMergeService(Service):
 
     @authentication_required
@@ -489,8 +446,8 @@ class MashupMergeService(Service):
     def process(self, request, to_ws_id):
 
         try:
-            data = json.loads(request.raw_post_data)
-        except ValueError, e:
+            data = json.loads(request.body)
+        except ValueError as e:
             msg = _("malformed json data: %s") % unicode(e)
             return build_error_response(request, 400, msg)
 
@@ -498,13 +455,13 @@ class MashupMergeService(Service):
         workspace_id = data.get('workspace', '')
 
         if mashup_id == '' and workspace_id == '':
-            return build_error_response(request, 422, _('missing workspace name or mashup id'))
+            return build_error_response(request, 422, _('Missing workspace or mashup parameter'))
         elif  mashup_id != '' and workspace_id != '':
-            return build_error_response(request, 422, _('missing workspace name or mashup id'))
+            return build_error_response(request, 422, _('Workspace and mashup parameters cannot be used at the same time'))
 
         to_ws = get_object_or_404(Workspace, id=to_ws_id)
         if not request.user.is_superuser and to_ws.creator != request.user:
-            return HttpResponseForbidden()
+            return build_error_response(request, 403, _('You are not allowed to update this workspace'))
 
         if mashup_id != '':
             values = mashup_id.split('/', 3)
@@ -524,31 +481,38 @@ class MashupMergeService(Service):
                 wgt_file = WgtFile(os.path.join(base_dir, resource.template_uri))
                 template = TemplateParser(wgt_file.get_template())
             else:
-                template = downloader.download_http_content(resource.template_uri, user=request.user)
+                template = download_http_content(resource.template_uri, user=request.user)
                 try:
                     template = TemplateParser(template)
                 except:
                     build_error_response(request, 424, _('Downloaded invalid resource description from: %(url)s') % {'url': resource.template_uri})
 
-            try:
-                check_mashup_dependencies(template, request.user)
-            except MissingDependencies, e:
-                details = {
-                    'missingDependencies': e.missing_dependencies,
-                }
-                return build_error_response(request, 422, unicode(e), details=details)
-
-            fillWorkspaceUsingTemplate(to_ws, template)
-
         else:
 
             from_ws = get_object_or_404(Workspace, id=workspace_id)
             if not request.user.is_superuser and from_ws.creator != request.user:
-                return HttpResponseForbidden()
+                return build_error_response(request, 403, _('You are not allowed to read from workspace %s') % workspace_id)
 
-            packageCloner = PackageCloner()
-            packageCloner.merge_workspaces(from_ws, to_ws, to_ws.creator)
+            options = {
+                'vendor': 'api',
+                'name': 'merge_op',
+                'version': '1.0',
+                'title': '',
+                'description': 'Temporal mashup for merging operation',
+                'email': 'a@example.com',
+            }
 
+            template = TemplateParser(build_json_template_from_workspace(options, from_ws, from_ws.creator))
+
+        try:
+            check_mashup_dependencies(template, request.user)
+        except MissingDependencies, e:
+            details = {
+                'missingDependencies': e.missing_dependencies,
+            }
+            return build_error_response(request, 422, unicode(e), details=details)
+
+        fillWorkspaceUsingTemplate(to_ws, template)
         return HttpResponse(status=204)
 
 
@@ -572,14 +536,14 @@ class WorkspacePublisherEntry(Resource):
         content_type = get_content_type(request)[0]
         image_file = None
         if content_type == 'application/json':
-            received_json = request.raw_post_data
+            received_json = request.body
         else:
             received_json = request.POST['json']
             image_file = request.FILES.get('image', None)
 
         try:
             options = json.loads(received_json)
-        except ValueError, e:
+        except ValueError as e:
             msg = _("malformed json data: %s") % unicode(e)
             return build_error_response(request, 400, msg)
 
@@ -599,7 +563,7 @@ class WorkspacePublisherEntry(Resource):
         workspace = get_object_or_404(Workspace, id=workspace_id)
         if image_file is not None:
             image_filename = 'images/catalogue' + os.path.splitext(image_file.name)[1]
-            options['image_uri'] = image_filename
+            options['image'] = image_filename
         description = build_rdf_template_from_workspace(options, workspace, request.user)
 
         f = StringIO()
